# [js 实现对树深度遍历与广度遍历](https://github.com/danygitgit/document-library)

**闲时要有吃紧的心思，忙时要有悠闲的趣味**

<a id="catalog">目录</a>

- [题目描述](#preface)
- [思路分析](#main-body)
- [AC 代码](#main-body2)

- [总结](#summary)

# <a  id="preface">题目描述</a>

> [返回目录](#catalog)

&emsp;js 实现对树深度遍历与广度遍历

# <a  id="main-body">思路分析</a>

## 深度遍历

&emsp;深度遍历是指从某个顶点出发，首先访问这个顶点，然后找出刚访问这个结点的第一个未被访问的邻结点，然后再以此邻结点为顶点，继续找它的下一个顶点进行访问。重复此步骤，直至所有结点都被访问完为止。

## 广度遍历

&emsp;广度遍历是从某个顶点出发，首先访问这个顶点，然后找出刚访问这个结点所有未被访问的邻结点，访问完后再访问这些结点中第一个邻结点的所有结点，重复此方法，直到所有结点都被访问完为止。

# <a  id="main-body2">AC 代码</a>

```js
//1.深度遍历的递归写法
function deepTraversal(node) {
  let nodes = []
  if (node != null) {
    nodes.push[node]
    let childrens = node.children
    for (let i = 0; i < childrens.length; i++) deepTraversal(childrens[i])
  }
  return nodes
}
```

```js
//2.深度遍历的非递归写法
function deepTraversal(node) {
  let nodes = []
  if (node != null) {
    let stack = [] //同来存放将来要访问的节点
    stack.push(node)
    while (stack.length != 0) {
      let item = stack.pop() //正在访问的节点
      nodes.push(item)
      let childrens = item.children
      for (
        let i = childrens.length - 1;
        i >= 0;
        i-- //将现在访问点的节点的子节点存入stack，供将来访问
      )
        stack.push(childrens[i])
    }
  }
  return nodes
}
```

```js
//3.广度遍历的递归写法
function wideTraversal(node) {
  let nodes = [],
    i = 0
  if (node != null) {
    nodes.push(node)
    wideTraversal(node.nextElementSibling)
    node = nodes[i++]
    wideTraversal(node.firstElementChild)
  }
  return nodes
}
```

```js
//4.广度遍历的非递归写法
function wideTraversal(node) {
  let nodes = [],
    i = 0
  while (node != null) {
    nodes.push(node)
    node = nodes[i++]
    let childrens = node.children
    for (let i = 0; i < childrens.length; i++) {
      nodes.push(childrens[i])
    }
  }
  return nodes
}
```

# <a  id="summary">总结</a>

> [返回目录](#catalog)

&emsp;立夏。加油！

[js 中二叉树的深度遍历与广度遍历(递归实现与非递归实现)](https://segmentfault.com/a/1190000016226334)

**后记：Hello 小伙伴们，如果觉得本文还不错，记得点个赞或者给个 star，你们的赞和 star 是我编写更多更丰富文章的动力！[GitHub 地址](https://github.com/danygitgit/document-library)**

# 文档协议

> <a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/4.0/"><img alt="知识共享许可协议" style="border-width:0" src="https://user-gold-cdn.xitu.io/2018/12/23/167d9537f3e29c99?w=88&h=31&f=png&s=1888" /></a><br /><a xmlns:dct="http://purl.org/dc/terms/" property="dct:title">**db** 的文档库</a> 由 <a xmlns:cc="http://creativecommons.org/ns#" href="db" property="cc:attributionName" rel="cc:attributionURL">db</a> 采用 <a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/4.0/">知识共享 署名-非商业性使用-相同方式共享 4.0 国际 许可协议</a>进行许可。<br />基于<a xmlns:dct="http://purl.org/dc/terms/" href="https://github.com/danygitgit" rel="dct:source">https://github.com/danygitgit</a>上的作品创作。<br />本许可协议授权之外的使用权限可以从 <a xmlns:cc="http://creativecommons.org/ns#" href="https://creativecommons.org/licenses/by-nc-sa/2.5/cn/" rel="cc:morePermissions">https://creativecommons.org/licenses/by-nc-sa/2.5/cn/</a> 处获得。
